"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("./util");
const child_process_1 = require("child_process");
const path = require("path");
const chalk = require("chalk");
const fs = require("fs-extra");
class Builders {
    static runTypescriptPostOps(dirs, component) {
        return __awaiter(this, void 0, void 0, function* () {
            const pkgJson = path.resolve(dirs.src, component, "package.json");
            const pkgJsonDest = path.resolve(dirs.out, component, "package.json");
            fs.copySync(pkgJson, pkgJsonDest);
            yield util_1.runCommand("npm", ["link"], path.resolve(dirs.out, component));
            // syncFolder(path.resolve(dirs.src, component, "node_modules"), path.resolve(dirs.out, component, "node_modules"));
        });
    }
    static buildCopy(dirs, component) {
        return __awaiter(this, void 0, void 0, function* () {
            util_1.log("Building " + chalk.magenta("copy") + " for " + chalk.cyan(component));
            const src = path.resolve(dirs.src, component);
            const dest = path.resolve(dirs.out, component);
            fs.copySync(src, dest, { overwrite: true });
            util_1.log("Building " + chalk.magenta("copy") + " for " + chalk.cyan(component) + "..." + chalk.green("done"));
        });
    }
    static buildTypescript(dirs, component) {
        return __awaiter(this, void 0, void 0, function* () {
            util_1.log("Building " + chalk.magenta("typescript") + " for " + chalk.cyan(component));
            const tscCmd = yield util_1.findBinCommand("tsc");
            yield new Promise((resolve, reject) => {
                let tsconfig = path.resolve(dirs.src, component, "tsconfig.json");
                let out = path.resolve(dirs.out, component);
                let args = ["-p", tsconfig, "--outDir", out];
                child_process_1.spawn(tscCmd, args, {
                    stdio: "inherit",
                    cwd: path.join(dirs.src, component)
                }).on("exit", function (code) {
                    if (code != 0)
                        return reject();
                    resolve();
                });
            });
            yield this.runTypescriptPostOps(dirs, component);
            util_1.log("Building " + chalk.magenta("typescript") + " for " + chalk.cyan(component) + "..." + chalk.green("done"));
        });
    }
    static buildRollup(dirs, component) {
        return __awaiter(this, void 0, void 0, function* () {
            util_1.log("Building " + chalk.magenta("rollup") + " for " + chalk.cyan(component));
            const rollupCmd = yield util_1.findBinCommand("rollup");
            yield new Promise((resolve, reject) => {
                let out = path.resolve(dirs.out, component);
                let conf = path.resolve(dirs.src, component, "rollup.config.js");
                child_process_1.spawn(rollupCmd, ["-c", conf, "-d", out], {
                    stdio: "inherit",
                    cwd: path.join(dirs.src, component)
                }).on("exit", function (code) {
                    if (code == 0)
                        return resolve();
                    let e = new Error("Unable to rollup '" + component + "'!");
                    e.showStack = false;
                    reject(e);
                });
            });
            yield this.runTypescriptPostOps(dirs, component);
            util_1.log("Building " + chalk.magenta("rollup") + " for " + chalk.cyan(component) + "..." + chalk.green("done"));
        });
    }
    static buildAngular(dirs, component) {
        return __awaiter(this, void 0, void 0, function* () {
            util_1.log("Building " + chalk.magenta("angular") + " for " + chalk.cyan(component));
            yield new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                const ngCmd = yield util_1.findBinCommand("ng");
                child_process_1.spawn(ngCmd, ["build", "--prod", "--outputPath=" + path.resolve(dirs.out, component)], {
                    stdio: "inherit",
                    cwd: path.resolve(dirs.src, component)
                }).on("exit", function (code) {
                    if (code != 0) {
                        let e = new Error("Unable to build angular for '" + component + "'!'");
                        e.showStack = false;
                        return reject(e);
                    }
                    resolve();
                });
            }));
            util_1.log("Building " + chalk.magenta("angular") + " for " + chalk.cyan(component) + "..." + chalk.green("done"));
        });
    }
}
exports.Builders = Builders;
