"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("../command");
const util_1 = require("../util");
const config_1 = require("./../config");
const chalk = require("chalk");
const builders_1 = require("./../builders");
class BuildCommand extends command_1.Command {
    doExec(args) {
        return __awaiter(this, void 0, void 0, function* () {
            let startTime = (new Date()).valueOf();
            let config = util_1.filterConfig(yield config_1.getConfig(), args);
            if (!config)
                return util_1.logError("No project found at " + process.cwd());
            util_1.log("Building project");
            let tasks = [];
            // TODO: Introduce and prebuild dependencies
            for (let component of config.components) {
                if (component.type == "angular") {
                    tasks.push(this.buildAngular(config, component));
                }
                else if (component.type == "typescript") {
                    tasks.push(this.buildTypescript(config, component));
                }
                else if (component.type == "rollup") {
                    tasks.push(this.buildRollup(config, component));
                }
                else if (component.type == "copy") {
                    tasks.push(this.buildCopy(config, component));
                }
                else {
                    throw new Error("Unknown build type for " + component.type);
                }
            }
            yield Promise.all(tasks);
            util_1.log("Building project..." + chalk.green("done"));
            let endTime = (new Date()).valueOf();
            let duration = Math.ceil((endTime - startTime) / 1000);
            util_1.log("Runtime: " + chalk.magenta(duration + " " + chalk.dim("seconds")));
        });
    }
    buildAngular(config, component) {
        return __awaiter(this, void 0, void 0, function* () {
            return builders_1.Builders.buildAngular(config.dirs, component.key);
        });
    }
    buildTypescript(config, component) {
        return __awaiter(this, void 0, void 0, function* () {
            return builders_1.Builders.buildTypescript(config.dirs, component.key);
        });
    }
    buildRollup(config, component) {
        return __awaiter(this, void 0, void 0, function* () {
            return builders_1.Builders.buildRollup(config.dirs, component.key);
        });
    }
    buildCopy(config, component) {
        return __awaiter(this, void 0, void 0, function* () {
            return builders_1.Builders.buildCopy(config.dirs, component.key);
        });
    }
    printHelp() {
        console.log(`
Synopsis

    spast build [component1] [component2] ... [options]

Build specific components or all of them if no component provided
    `);
    }
}
exports.BuildCommand = BuildCommand;
