"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("../command");
const util_1 = require("../util");
const inquirer = require("inquirer");
const path = require("path");
const fs = require("fs");
const config_1 = require("./../config");
class CreateCommand extends command_1.Command {
    validateKey(config, key) {
        if (!key)
            return "Missing key";
        if (!/^[a-zA-Z][a-zA-Z0-9]*$/.exec(key))
            return "Invalid key! Only characters and numbers are allowed!";
        return true;
    }
    doExec(args) {
        return __awaiter(this, void 0, void 0, function* () {
            let config = yield config_1.getConfig();
            if (config)
                return util_1.logError("A project already exists at " + process.cwd());
            util_1.log("Creating new project");
            let folderName = path.basename(process.cwd());
            let answers = yield inquirer.prompt([
                { type: 'input', name: 'name', message: 'Name', required: true, validate: (i) => this.validateKey(config, i), default: folderName },
            ]);
            config = {
                name: answers.name,
                dirs: {
                    src: './src',
                    out: './build'
                },
                components: [],
                createdAt: new Date().getTime()
            };
            yield config_1.setConfig(config);
            let pkg = {
                name: answers.name,
                private: true,
                version: "0.0.1",
                author: "",
                description: "",
                repository: "",
                license: "",
            };
            fs.writeFileSync("package.json", JSON.stringify(pkg, null, 2), { encoding: "utf8" });
            yield util_1.runCommand("npm", ["i", "typescript", "--save-dev"]);
        });
    }
    printHelp() {
        console.log(`
Synopsis

    spast create [options]

Creates a new SPAST project

Options
#    -d, --verbose ...................... debug mode produces verbose log output for all activity,

Examples
   !!! TODO
    `);
    }
}
exports.CreateCommand = CreateCommand;
