"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("../command");
const util_1 = require("../util");
const config_1 = require("./../config");
const path = require("path");
const chalk = require("chalk");
const fs = require("fs-extra");
class InstallCommand extends command_1.Command {
    doExec(args) {
        return __awaiter(this, void 0, void 0, function* () {
            let startTime = (new Date()).valueOf();
            let config = util_1.filterConfig(yield config_1.getConfig(), args);
            if (!config)
                return util_1.logError("No project found at " + process.cwd());
            util_1.log("Installing project dependnecies");
            let tasks = [];
            for (let component of config.components) {
                tasks.push(this.install(config.dirs, component));
            }
            yield Promise.all(tasks);
            util_1.log("Installing project dependencies..." + chalk.green("done"));
            let endTime = (new Date()).valueOf();
            let duration = Math.ceil((endTime - startTime) / 1000);
            util_1.log("Runtime: " + chalk.magenta(duration + " " + chalk.dim("seconds")));
        });
    }
    install(dirs, component) {
        return __awaiter(this, void 0, void 0, function* () {
            const pkgPath = path.resolve(dirs.src, component.key);
            const pkgJson = path.join(pkgPath, "package.json");
            if (fs.existsSync(pkgJson)) {
                util_1.log("Installing dependencies for " + chalk.cyan(component.key));
                yield util_1.runCommand("npm", ["install"], pkgPath);
            }
        });
    }
    printHelp() {
        console.log(`
Synopsis

    spast install [component1] [component2] ... [options]

Install project dependencies for specific components or all of them if no component provided
    `);
    }
}
exports.InstallCommand = InstallCommand;
