"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("../command");
const util_1 = require("../util");
const config_1 = require("./../config");
const chalk = require("chalk");
const starters_1 = require("./../starters");
class StartCommand extends command_1.Command {
    doExec(args) {
        return __awaiter(this, void 0, void 0, function* () {
            let config = util_1.filterConfig(yield config_1.getConfig(), args);
            if (!config)
                return util_1.logError("No project found at " + process.cwd());
            util_1.log("Starting project");
            const libs = {};
            let tasks = [];
            for (let component of config.components) {
                if (!component.dependencies)
                    continue;
                for (let dependency of component.dependencies) {
                    if (!libs[dependency])
                        libs[dependency] = [];
                    libs[dependency].push(component.key);
                }
            }
            let components = [...config.components];
            for (let lib in libs) {
                if (config.components.find(i => i.key == lib))
                    continue;
                components.unshift(config._unfiltered.components.find(j => j.key == lib));
            }
            for (let component of components) {
                if (component.type == "angular") {
                    tasks.push(this.startAngular(config, component, libs));
                }
                else if (component.type == "typescript") {
                    tasks.push(this.startTypescript(config, component, libs));
                }
                else if (component.type == "rollup") {
                    tasks.push(this.startRollup(config, component, libs));
                }
                else if (component.type == "copy") {
                    tasks.push(this.startCopy(config, component, libs));
                }
                else {
                    throw new Error("Unknown build type for " + component.type);
                }
            }
            yield Promise.all(tasks);
            util_1.log("Starting project..." + chalk.green("done"));
            // Keep open
            yield new Promise((resolve, reject) => {
                process.on("exit", () => resolve());
            });
        });
    }
    startAngular(config, component, libs) {
        return __awaiter(this, void 0, void 0, function* () {
            return starters_1.Starters.startAngular(config.dirs, component);
        });
    }
    startTypescript(config, component, libs) {
        return __awaiter(this, void 0, void 0, function* () {
            return starters_1.Starters.startTypescript(config.dirs, component);
        });
    }
    startRollup(config, component, libs) {
        return __awaiter(this, void 0, void 0, function* () {
            return starters_1.Starters.startRollup(config.dirs, component);
        });
    }
    startCopy(config, component, libs) {
        return __awaiter(this, void 0, void 0, function* () {
            return starters_1.Starters.startCopy(config.dirs, component);
        });
    }
    printHelp() {
        console.log(`
Synopsis

    spast start [component1] [component2] ... [options]

Start a live reloadable instance of specific components or all of them if no component provided
    `);
    }
}
exports.StartCommand = StartCommand;
