"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fs = require("fs");
const chalk = require("chalk");
const util_1 = require("./util");
var ComponentType;
(function (ComponentType) {
    ComponentType["typescript"] = "typescript";
    ComponentType["angular"] = "angular";
    ComponentType["rollup"] = "rollup";
    ComponentType["copy"] = "copy";
})(ComponentType = exports.ComponentType || (exports.ComponentType = {}));
function getConfig(dir) {
    if (!dir)
        dir = process.cwd();
    let configPath = path.join(dir, "spast.json");
    if (!fs.existsSync(configPath))
        return null;
    util_1.logDebug("Retrieving " + chalk.cyan("spast.json") + " from disk");
    let configData = require(configPath);
    let depsWeight = {};
    for (let component of configData.components) {
        if (!depsWeight[component.key])
            depsWeight[component.key] = 0;
        depsWeight[component.key]++;
        if (component.dependencies) {
            for (let dep of component.dependencies) {
                if (!depsWeight[dep])
                    depsWeight[dep] = 0;
                depsWeight[dep]++;
            }
        }
    }
    configData.components.sort((a, b) => {
        let as = depsWeight[a.key];
        let bs = depsWeight[b.key];
        if (bs > as)
            return 1;
        if (bs < as)
            return -1;
        return 0;
    });
    configData._unfiltered = JSON.parse(JSON.stringify(configData));
    return configData;
}
exports.getConfig = getConfig;
function setConfig(data, dir) {
    if (!dir)
        dir = process.cwd();
    let configPath = path.join(dir, "spast.json");
    let writeData = JSON.parse(JSON.stringify(data));
    delete data._unfiltered;
    util_1.logDebug("Writing new config to " + chalk.cyan("spast.json"));
    fs.writeFileSync(configPath, JSON.stringify(writeData, null, 2), { encoding: "utf8" });
}
exports.setConfig = setConfig;
