"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("./util");
const child_process_1 = require("child_process");
const path = require("path");
const chalk = require("chalk");
const chokidar = require("chokidar");
const builders_1 = require("./builders");
let typescriptProcesses = {};
class Starters {
    static runTypescriptProcess(dirs, component) {
        return __awaiter(this, void 0, void 0, function* () {
            let scriptPath = path.resolve(dirs.out, component.key, component.script);
            if (typescriptProcesses[scriptPath]) {
                let runner = typescriptProcesses[scriptPath];
                if (runner.tm)
                    clearTimeout(runner.tm);
                if (runner.process) {
                    runner.process.removeListener('exit', runner.exitHandler);
                    runner.process.kill(9);
                    util_1.log("Killed process " + chalk.magentaBright(runner.process.pid) + " for " + chalk.cyan(component.key));
                }
            }
            let handler = (code, signal) => {
                if (code != null) {
                    if (code != 0) {
                        util_1.logError("Process " + chalk.magenta(component.script) + " for " + chalk.cyan(component.key) + " has crashed with code " + code + '". Restarting in 5 seconds...!\n');
                    }
                    else {
                        util_1.log("Clean exit of process " + chalk.magenta(component.script) + " for " + chalk.cyan(component.key));
                    }
                    typescriptProcesses[scriptPath].tm = setTimeout(() => Starters.runTypescriptProcess(dirs, component), 5000);
                }
                else {
                    typescriptProcesses[scriptPath].tm = setTimeout(() => Starters.runTypescriptProcess(dirs, component), 1000);
                }
            };
            util_1.log("Starting script " + chalk.magenta(component.script) + " for " + chalk.cyan(component.key));
            let child = child_process_1.fork(scriptPath);
            child.on('exit', handler);
            typescriptProcesses[scriptPath] = { exitHandler: handler, process: child };
        });
    }
    static reloadTypescriptProcess(dirs, component) {
        return __awaiter(this, void 0, void 0, function* () {
            util_1.log("Reloading " + chalk.magenta("typescript") + " for " + chalk.cyan(component.key));
            return Starters.runTypescriptProcess(dirs, component);
        });
    }
    static startCopy(dirs, component) {
        return __awaiter(this, void 0, void 0, function* () {
            util_1.log("Starting " + chalk.magenta("copy") + " for " + chalk.cyan(component.key));
            const src = path.resolve(dirs.src, component.key);
            const dest = path.resolve(dirs.out, component.key);
            yield util_1.syncFolder(src, dest, null, true);
        });
    }
    static startTypescript(dirs, component) {
        return __awaiter(this, void 0, void 0, function* () {
            util_1.log("Starting " + chalk.magenta("typescript") + " for " + chalk.cyan(component.key));
            yield builders_1.Builders.buildTypescript(dirs, component.key);
            let c = chokidar.watch(path.resolve(dirs.src, component.key, '**', '*.ts'), {
                persistent: true,
                ignored: /node_modules/
            });
            let reload = Starters.reloadTypescriptProcess;
            let notifyTm;
            function onWatchedNotify() {
                return __awaiter(this, void 0, void 0, function* () {
                    if (notifyTm)
                        clearTimeout(notifyTm);
                    notifyTm = setTimeout(() => __awaiter(this, void 0, void 0, function* () {
                        yield builders_1.Builders.buildTypescript(dirs, component.key);
                        reload(dirs, component);
                    }), 150);
                });
            }
            setTimeout(() => c.on('all', (e, p) => __awaiter(this, void 0, void 0, function* () { return onWatchedNotify(); })), 2500);
            if (component.script)
                yield onWatchedNotify();
        });
    }
    static startRollup(dirs, component) {
        return __awaiter(this, void 0, void 0, function* () {
            util_1.log("Starting " + chalk.magenta("rollup") + " for " + chalk.cyan(component.key));
            yield builders_1.Builders.buildRollup(dirs, component.key);
            let c = chokidar.watch(path.resolve(dirs.src, component.key, '**', '*.ts'), {
                persistent: true,
                ignored: /node_modules/
            });
            let notifyTm;
            function onWatchedNotify(p) {
                return __awaiter(this, void 0, void 0, function* () {
                    if (notifyTm)
                        clearTimeout(notifyTm);
                    notifyTm = setTimeout(() => __awaiter(this, void 0, void 0, function* () {
                        yield builders_1.Builders.buildRollup(dirs, component.key);
                    }), 150);
                });
            }
            setTimeout(() => c.on('all', (e, p) => __awaiter(this, void 0, void 0, function* () { return onWatchedNotify(p); })), 2500);
        });
    }
    static startAngular(dirs, component) {
        return __awaiter(this, void 0, void 0, function* () {
            util_1.log("Starting " + chalk.magenta("angular") + " for " + chalk.cyan(component.key));
            let cwd = path.resolve(dirs.src, component.key);
            yield new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                const ngCmd = yield util_1.findBinCommand("ng");
                let args = ["serve"];
                if (component.port) {
                    args.push("--port");
                    args.push(component.port);
                }
                /*, "--host", component.url.development*/
                let pc = child_process_1.spawn(ngCmd, args, {
                    stdio: 'inherit',
                    cwd: cwd
                });
                pc.on("exit", function (code) {
                    if (code != 0) {
                        let e = new Error("Unable to start angular for '" + component.key + "'!'");
                        e.showStack = false;
                        return reject(e);
                    }
                    resolve();
                });
            }));
        });
    }
}
exports.Starters = Starters;
