"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const chalk = require("chalk");
const fs = require("fs");
const minimist = require("minimist");
const child_process_1 = require("child_process");
let parsedArgs;
function printHeader() {
    const m = chalk.magentaBright;
    const c = chalk.black;
    const g = chalk.cyan;
    const y = chalk.black;
    const w = chalk.white;
    const u = chalk.bold;
    const b = chalk.magenta;
    console.log(g('OoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoO'));
    console.log(g('8        ' + m('____  ___   ____  ____  ___  ') + '        8'));
    console.log(g('O        ' + m('[__   |__]  |__|  [__    |   ') + '        O'));
    console.log(g('8        ' + m('___] .|    .|  | .___] . |  .') + '        8'));
    console.log(g('O                                             O'));
    console.log(g('O        ' + w(b(u('S')) + 'oftware ' + b(u('P')) + 'roject ' + b(u('A')) + 'ssistant to') + '        O'));
    console.log(g('O          ' + w(b(u('S')) + 'horten Development ' + b(u('T')) + 'ime') + '           O'));
    // console.log(g('8                  ' + mg(' ' + c('v') + y(version) + ' ') + '                   8'));
    console.log(g('OoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoO'));
    console.log("");
}
exports.printHeader = printHeader;
function getVersion() {
    let pkg = require(path.join(__dirname, "package.json"));
    let version = pkg.version;
    return version;
}
exports.getVersion = getVersion;
function printVersion() {
    let version = getVersion();
    console.log(version);
}
exports.printVersion = printVersion;
function parseArgs() {
    if (parsedArgs)
        return parsedArgs;
    parsedArgs = minimist(process.argv.slice(2));
    return parsedArgs;
}
exports.parseArgs = parseArgs;
function generateMessage(msg) {
    if (msg instanceof Error) {
        return msg.message + "\n" + msg.stack;
    }
    else if (typeof msg === "object") {
        return JSON.stringify(msg, null, 2);
    }
    return String(msg);
}
function padLeft(nr, n, str) {
    return Array(n - String(nr).length + 1).join(str || '0') + nr;
}
exports.padLeft = padLeft;
function timestamp() {
    let now = new Date();
    let timestamp = this.padLeft(now.getHours(), 2) + ":" + this.padLeft(now.getMinutes(), 2) + ":" + this.padLeft(now.getSeconds(), 2);
    return "[" + timestamp + "]";
}
exports.timestamp = timestamp;
function log(msg) {
    let _msg = generateMessage(msg);
    process.stdout.write(chalk.gray(this.timestamp()) + " " + chalk.yellow(_msg) + "\n");
}
exports.log = log;
function logError(msg) {
    let _msg = chalk.red(generateMessage(msg));
    process.stderr.write(chalk.gray(this.timestamp()) + " " + _msg + "\n");
}
exports.logError = logError;
function logDebug(msg) {
    let args = parseArgs();
    if (!args.d && !args.verbose)
        return;
    let _msg = chalk.green(generateMessage(msg));
    process.stderr.write(chalk.gray(this.timestamp()) + " " + _msg + "\n");
}
exports.logDebug = logDebug;
function syncFolder(src, dest, cwd, watch = false) {
    return __awaiter(this, void 0, void 0, function* () {
        yield new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            let cmd = yield findBinCommand("sync-folders");
            let args = ["--ignore", "/node_modules/", "--ignore", "/\.DS_Store/", "--ignore", "Thumbs\.db",
                "--verbose",
            ];
            if (watch)
                args.push("--watch");
            child_process_1.spawn(cmd, [...args, src, dest], {
                stdio: "inherit",
                cwd: cwd || process.cwd()
            }).on("exit", function (code) {
                if (code != 0) {
                    let e = new Error("Unable to sync folder from '" + src + "' to '" + dest + "'!");
                    e.showStack = false;
                    return reject(e);
                }
                resolve();
            });
        }));
    });
}
exports.syncFolder = syncFolder;
function runCommand(command, args, cwd, silent = false) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            let stdout = "";
            let proc = child_process_1.spawn(command, args, { cwd: cwd || process.cwd() });
            proc.stdout.on("data", (d) => {
                stdout += d;
                if (!silent)
                    process.stdout.write(d);
            });
            proc.stderr.on("data", (d) => !silent && process.stderr.write(d));
            proc.on("close", (code) => {
                if (code == 0)
                    return resolve(stdout);
                return reject();
            });
        });
    });
}
exports.runCommand = runCommand;
function findBinCommand(command) {
    return __awaiter(this, void 0, void 0, function* () {
        let pathsToCheck = [...module.paths];
        pathsToCheck.push(yield runCommand("npm", ["root", "--global"], null, true));
        pathsToCheck.push(path.join(process.cwd(), "node_modules"));
        for (let p of pathsToCheck) {
            let cmd = path.join(p, ".bin", command);
            try {
                if (!fs.existsSync(cmd))
                    continue;
                fs.accessSync(cmd, fs.constants.X_OK);
                return cmd;
            }
            catch (err) {
            }
        }
        throw new Error("No binary found for command " + command);
    });
}
exports.findBinCommand = findBinCommand;
function filterConfig(config, argv) {
    const packageNames = argv._.slice(1);
    if (packageNames.length > 0) {
        let filteredPackages = [];
        for (let pkg of packageNames) {
            let found = config.components.find(i => i.key == pkg);
            if (!found)
                logError("Package " + pkg + " not found");
            else
                filteredPackages.push(found);
        }
        config.components = filteredPackages;
    }
    return config;
}
exports.filterConfig = filterConfig;
